/*
 * linux/arch/arm/mach-pxa/kgdb-serial.c
 *
 * Provides low level kgdb serial support hooks for PXA2xx boards
 *
 * Author:	Nicolas Pitre
 * Copyright:	(C) 2002-2005 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <bsp.h>                /* Must be before libio.h */
#include <rtems/libio.h>
#include <termios.h>
#include <rtems/bspIo.h>

#include "gdb-stub.h"

/* Put the CPU (or UART) specific header file #include here */
#include <mc9328mxl.h>
#include <irqsupport.h>
#include <mx1_reg.h>

/* call set_debug_traps() once you are ready for the stub to start. */

int kgdb_debug_channel = 1;
int kgdb_debug_baudrate = 19200;

int kgdb_serial_set_parameters(int channel, int baudrate)
{
	if(channel != -1){
		kgdb_debug_channel = channel;
	}
	if(baudrate != -1){
		kgdb_debug_baudrate = baudrate;
	}
	/*imx_uart_set_baud(kgdb_debug_channel, baudrate);*/
	return 0;
}

static int kgdb_serial_init(void)
{
	/*imx_uart_set_baud(kgdb_debug_channel, kgdb_debug_baudrate);*/
	return 1;
}

static void kgdb_serial_putchar(int c)
{
	imx_uart_poll_write_char(kgdb_debug_channel, c);
}

static void kgdb_serial_flush(void)
{
}

static int kgdb_serial_getchar(void)
{
	int val;

	do {
		val = imx_uart_poll_read_char(kgdb_debug_channel);
#if 0
		_reg_WDG_WSR=MX1_WDG_WSR_PHA1_m;
		_reg_WDG_WSR=MX1_WDG_WSR_PHA2_m;
#endif
	} while (val == -1);

	return val;
}

struct kgdb_io kgdb_io_ops = {
	.init		= kgdb_serial_init,
	.write_char	= kgdb_serial_putchar,
	.flush		= kgdb_serial_flush,
	.read_char	= kgdb_serial_getchar,
};

void kgdb_do_fiq_exception(int excptnum, struct pt_regs *regs)
{
	char ch;
	ch = imx_uart_poll_read_char(kgdb_debug_channel);
	if(ch==3)
		kgdb_handle_exception(excptnum, SIGINT, 0, regs);
}

void set_debug_traps(void)
{
	unsigned long flags;
	volatile struct mx1_uartx_regs *puart;
	int uart_rx_irq;

	fiq_save_and_cli(flags);

	kgdb_init();

	if(kgdb_debug_channel==1){
		uart_rx_irq=MX1_UART2_MINT_RX;
		puart=&_reg_UART2;
	}else{
		uart_rx_irq=MX1_UART1_MINT_RX;
		puart=&_reg_UART1;
	}

	set_irq_priority(uart_rx_irq, IRQ_PRIORITY_LEVEL_FIQ);
	enable_irq(uart_rx_irq);

	puart->UFCR&=~MX1_UX_UFCR_RXTL_m;
	puart->UFCR|=__val2mfld(MX1_UX_UFCR_RXTL_m,1);
	puart->UCR1|=MX1_UX_UCR1_RRDYEN_m;

	_reg_AITC_INTCNTL|=MX1_AITC_INTCNTL_FIAD_m;
	//_reg_AITC_INTCNTL|=MX1_AITC_INTCNTL_NIAD_m;

	set_cpu_exception_handler(ARM_EXCEPTION_FIQ, kgdb_do_fiq_exception);
	restore_flags(flags);
}

int breakpoint_command_forshell(int argc, char **argv)
{
  unsigned long flags;
  fiq_save_and_cli(flags);
 #if 0
  asm (
    "   mov     r0,#0x100\n\t"
    "   mov     r1,#0x11\n\t"
    "   mov     r2,#0x22\n\t"
    "   mov     r3,#0x33\n\t"
    "   mov     r4,#0x44\n\t"
    "   mov     r5,#0x55\n\t"
    "   mov     r6,#0x66\n\t"
    "   mov     r7,#0x77\n\t"
    "   mov     r8,#0x88\n\t"
    "   mov     r9,#0x99\n\t"
    "   mov     r10,#0xaa\n\t"
    "   mov     r11,#0xbb\n\t"
    "   mov     r12,#0xcc\n\t"
    "   mov     r13,#0xdd\n\t"
    "   mov     r14,#0xee\n"
    "   .word   0xe7ffdeff\n"
    "   mov     r0,#0x0a\n\t"
    "   mov     r1,#0x1a\n\t"
    "   mov     r2,#0x2a\n\t"
    "   .word   0xe7ffdeff\n"
    "1: b       1b\n\t"
  );
 #else
  asm (
    "   .word   0xe7ffdeff\n"
  );
 #endif
 return 0;
}
